#include "designingarea.h"
//#include "selectionsquares.h"
#include <QPainter>
#include <QDragEnterEvent>
#include <QDragMoveEvent>
#include <QDropEvent>
#include <QContextMenuEvent>
#include <QMessageBox>
#include <qmath.h>
#include <QDebug>
#include "widgetfactory.h"
#include "constants.h"
#include "props_functions.h"

#include "Q7BFormDesigner.h"

extern Q7BFormDesigner *GLOBAL_Q7BFormDesigner;

DesigningArea::DesigningArea (QWidget *parent)
    : QFrame (parent)
{
    setAcceptDrops (true);
    setFrameShape ( QFrame::Box );
    setLineWidth ( 0 ) ;
    movingWidget = 0 ;
    drawingSelection = false ;
    setMouseTracking ( true ) ;
    setGrid () ;
    setShowCursorPos ( true ) ;

    PF::setPropReadOnly ( this , QString ( "objectName" ) ) ;
    PF::setPropReadOnly ( this , QString ( "enabled" ) ) ;
    mode = 0 ;
}

const int DesigningArea::arrowMovingDelta = 20 ;

const int DesigningArea::arrowResizingDelta = 10 ;

void DesigningArea::setGrid ( int delta )
{
    if( delta <= 0 )
        return ;
    gridDelta = delta ;

    foreach ( QObject * w , children () )
    {
        //if ( w->property ( SEL_SQUARES_MARK ).toBool () == true )
        if( PF::readProperty( w , SEL_SQUARES_MARK ).toBool () == true )
            continue ;
        else
        {
            QWidget * widget = qobject_cast< QWidget * > ( w ) ;
            if ( widget )
                widget->setGeometry ( rectToGrid ( widget->geometry () ) ) ;
        }
    }
    update () ;
}

void DesigningArea::dragEnterEvent (QDragEnterEvent *event)
{
    if ( mode )
        return ;
    if  (event->mimeData ()->hasText () )
    {
        if ( ! childAt ( event->pos () ) )
        {
            event->acceptProposedAction () ;
            event->setDropAction (Qt::CopyAction);
        }
        else
        {
            if ( childAt ( event->pos () )->acceptDrops () )
            {
                event->acceptProposedAction () ;
            }
            else
            {
                event->ignore () ;
            }
        }
    }
    else
        event->ignore ();
}

void DesigningArea::dragMoveEvent ( QDragMoveEvent * event )
{
    if ( mode )
        return ;
    if ( ! childAt ( event->pos () ) )
    {
        event->acceptProposedAction () ;
        event->setDropAction (Qt::CopyAction);

    }
    else
    {
        if ( childAt ( event->pos () )->acceptDrops () )
            event->acceptProposedAction () ;
        else
            event->ignore () ;
    }
}

void DesigningArea::dropEvent (QDropEvent *event)
{
    if ( mode )
        return ;
    if  (event->mimeData ()->hasText ())
    {
        QPoint hotSpot;

        QList<QByteArray> hotSpotPos = event->mimeData ()->data ("application/x-hotspot").split (' ');
        if  (hotSpotPos.size () == 2) {
            hotSpot.setX (hotSpotPos.first ().toInt ());
            hotSpot.setY (hotSpotPos.last ().toInt ());
        }

        event->setDropAction ( Qt::MoveAction ) ;
        event->accept ();
        emit inFocus ( this ) ;
        emit clearSelection () ;
        emit moveWidgetByPaste ( childAt ( event->pos () ) ? childAt ( event->pos () ) : this ,
                                hotSpot ) ;
    }
    else
        event->ignore ();
}

void DesigningArea::mousePressEvent (QMouseEvent *event)
{
    QFrame::mousePressEvent ( event ) ;
    if ( mode )
        return ;

    setFocus ();
    emit inFocus ( this ) ;

    if ( event->button () == Qt::LeftButton )
    {
        //if ( childAt ( event->pos () ) && childAt ( event->pos () )->property ( SEL_SQUARES_MARK ).toBool () == true )
        if ( childAt ( event->pos () ) && childAt ( event->pos () )->property( SEL_SQUARES_MARK ).toBool () == true )
            return ;
        movingWidget = qobject_cast < QWidget * >  ( childAt ( event->pos () ) ) ;
        if (movingWidget)
        {
            movingWidget = WidgetFactory::instance()->findKnownParent(movingWidget);
        }
        if  ( ! movingWidget )
        {
            startSelectionPos = event->pos () ;
            endSelectionPos = event->pos () ;
            drawingSelection = true ;
            emit clearSelection () ;
            moving = false ;
        }
        else
        {
            prevMovingPos = event->pos () ;
            prevMovingPos = pointToGrid ( prevMovingPos ) ;
            if ( event->buttons ().testFlag ( Qt::LeftButton ) )
                moving = true ;
            else
                moving = false ;
            setCursor ( QCursor ( Qt::ClosedHandCursor ) ) ;
            //movingWidget->raise () ;
        }
        event->accept () ;
    }
    else if ( event->button () == Qt::RightButton )
    {
        moving = false ;
        movingWidget = qobject_cast < QWidget * >  ( childAt ( event->pos () ) ) ;
        movingWidget = WidgetFactory::instance()->findKnownParent(movingWidget);
        if (movingWidget)
        {
            emit widgetsSelected ( QList< QPointer < QWidget > > () << movingWidget ,
                                  false ) ;
            emit contextMenuSignal ( movingWidget ) ;
        }
    }
    emit leftClicked () ;
}

int DesigningArea::getGridDelta()
{
    if( gridDelta > 0 )
        return gridDelta ;
    else
        return 1 ;
}

QPoint DesigningArea::pointToGrid ( QPoint p )
{
    if ( gridDelta > 0 )
    {

        if ( qAbs ( double ( p.x () % gridDelta ) ) / double ( gridDelta ) > 0.5  )
        {
            if ( p.x () > 0 )
                p.setX (  ( p.x () + gridDelta ) / gridDelta  * gridDelta ) ;
            else
                p.setX (  ( p.x () - gridDelta ) / gridDelta  * gridDelta ) ;
        }
        else
            p.setX ( p.x () / gridDelta  * gridDelta ) ;

        if ( qAbs ( double ( p.y () % gridDelta ) ) / double ( gridDelta ) > 0.5 )
        {
            if ( p.y () > 0 )
                p.setY (  ( p.y () + gridDelta ) / gridDelta  * gridDelta ) ;
            else
                p.setY (  ( p.y () - gridDelta ) / gridDelta  * gridDelta ) ;
        }
        else
            p.setY ( p.y () / gridDelta  * gridDelta ) ;
    }
    return p ;
}

void DesigningArea::mouseReleaseEvent ( QMouseEvent * event )
{
    if ( mode )
        return ;
    if ( ! movingWidget )
    {
        drawingSelection = false ;

        QList< QWidget * > curSelection ;
        QWidget * widget ;
        foreach ( QObject * w , children () )
        {
            if ( w->property( SEL_SQUARES_MARK ).toBool () == true )
                continue ;
            else
            {
                widget = qobject_cast< QWidget * > ( w ) ;
                if ( widget  )
                {
                    if ( QRect ( startSelectionPos , endSelectionPos ).intersects ( widget->geometry () ) )
                        if (  ! curSelection.contains ( widget ) )
                            curSelection.append ( widget ) ;
                }
            }
        }
        QList< QPointer < QWidget > > sel ;
        foreach ( QWidget * w , curSelection )
            sel << w ;
        if ( ! sel.isEmpty () )
            emit widgetsSelected ( sel , event->modifiers ().testFlag ( Qt::ControlModifier ) ) ;
        else
            emit widgetsSelected ( QList< QPointer < QWidget > > () << this , false ) ;
        update () ;
    }
    else
        emit widgetsSelected ( QList< QPointer < QWidget > > () << movingWidget ,
                              event->modifiers ().testFlag ( Qt::ControlModifier ) ) ;
    setCursor ( QCursor ( Qt::ArrowCursor ) ) ;
    movingWidget = 0 ;
}

void DesigningArea::mouseMoveEvent ( QMouseEvent * event)
{
    if ( mode )
        return ;
    emit inFocus ( this ) ;
    if ( showCursorPos )
        setToolTip ( QString ( "X:%1 Y:%2" ).arg ( QString::number ( event->pos ().x () ) ).arg ( QString::number ( event->pos ().y () ) ) ) ;
    else
        setToolTip ( "" ) ;
    //curMousePos = event->pos () ;
    if ( movingWidget && moving )
    {
        moving = false ;

        QPoint newPos = event->pos () ;
        newPos = pointToGrid ( newPos ) ;
        prevMovingPos = newPos ;

        emit widgetsDragged ( movingWidget ) ;
    }
    else if (  ( event->buttons () & Qt::LeftButton ) && drawingSelection )
    {
        endSelectionPos = event->pos () ;
        setCursor ( QCursor ( Qt::CrossCursor ) ) ;
        update () ;
    }
    else
        setCursor ( QCursor ( Qt::ArrowCursor ) ) ;
}

void DesigningArea::paintEvent (QPaintEvent * e)
{
    QFrame::paintEvent ( e ) ;
    if ( mode )
        return ;
    QPainter painter ( this ) ;
    if ( gridDelta > 1 )
    {
        QPolygon polygon ;
        for ( int i = 0 ; i < width () ; i += gridDelta )
            for ( int j = 0 ; j < height () ; j += gridDelta )
                polygon << QPoint ( i , j ) ;
        painter.setPen ( QPen ( QBrush (  Qt::black ) , 1 ) ) ;
        painter.drawPoints ( polygon ) ;
    }
    if ( drawingSelection )
    {
        painter.setPen ( QPen ( QBrush (  Qt::black ) , 1 , Qt::DotLine ) ) ;
        QRect rect (startSelectionPos , endSelectionPos ) ;
        painter.drawRect ( rect ) ;
    }
}

bool DesigningArea::eventFilter ( QObject * watched, QEvent * event )
{
    if ( QString ( watched->metaObject()->className () ) == QString ( "QLineEdit" ) )
        qDebug ( ) << event->type () ;
    if ( mode )
        return false ;
    if ( event->type () == QEvent::MouseButtonRelease )
    {
        QMouseEvent * me = static_cast< QMouseEvent * > ( event ) ;
        if ( watched->property( SEL_SQUARES_MARK ).toBool () == true )
            return true ;
        QWidget * w = qobject_cast< QWidget * > ( watched ) ;

        if ( w )
        {
            QPoint pos = me->pos () + w->pos () ;
            QWidget * par = w->parentWidget () ;
            while ( par != 0 && par != this )
            {
                pos += par->pos () ;
                par = par->parentWidget () ;
            }
            mouseReleaseEvent ( new QMouseEvent ( me->type () ,
                                              pos ,
                                              me->button () ,
                                              me->buttons () ,
                                              me->modifiers ()) ) ;
            return true ;
        }
    }
    else if ( event->type () == QEvent::MouseButtonPress )
    {
        QMouseEvent * me = static_cast< QMouseEvent * > ( event ) ;
        if ( watched->property( SEL_SQUARES_MARK ).toBool () == true )
            return true ;
        QWidget * w = qobject_cast< QWidget * > ( watched ) ;

        if (  ( me->buttons () &  ( Qt::LeftButton | Qt::RightButton ) ) && w )
        {
            QPoint pos = me->pos () + w->pos () ;
            QWidget * par = w->parentWidget () ;
            while ( par != 0 && par != this )
            {
                pos += par->pos () ;
                par = par->parentWidget () ;
            }
            mousePressEvent ( new QMouseEvent ( me->type () ,
                                              pos ,
                                              me->button () ,
                                              me->buttons () ,
                                              me->modifiers ()) ) ;
            return true ;
        }

    }
    else if ( event->type () == QEvent::MouseMove )
    {

        QMouseEvent * me = static_cast< QMouseEvent * > ( event ) ;
        QWidget * w = qobject_cast< QWidget * > ( watched ) ;
        if ( w )
        {
            QPoint pos = me->pos () + w->pos () ;
            mouseMoveEvent ( new QMouseEvent ( me->type () ,
                                             pos ,
                                             me->button () ,
                                             me->buttons () ,
                                             me->modifiers () ) ) ;
        }
        return true ;
    }
    else if ( event->type () == QEvent::KeyPress )
    {
        QKeyEvent * ke = static_cast< QKeyEvent * > ( event ) ;
        if ( ke->key() == Qt::Key_Right
             || ke->key() == Qt::Key_Left
             || ke->key() == Qt::Key_Up
             || ke->key() == Qt::Key_Down )
        {
            keyPressEvent ( new QKeyEvent ( ke->type () ,
                                          ke->key () ,
                                          ke->modifiers () ,
                                          ke->text () ,
                                          ke->isAutoRepeat () ,
                                          ke->count () ) ) ;
            return true ;
        }
        else
            return false ;
    }
    else if ( event->type () == QEvent::KeyRelease )
    {
        QKeyEvent * ke = static_cast< QKeyEvent * > ( event ) ;
        if ( ke->key() == Qt::Key_Right
             || ke->key() == Qt::Key_Left
             || ke->key() == Qt::Key_Up
             || ke->key() == Qt::Key_Down )
        {
            keyReleaseEvent ( new QKeyEvent ( ke->type () ,
                                            ke->key () ,
                                            ke->modifiers () ,
                                            ke->text () ,
                                            ke->isAutoRepeat () ,
                                            ke->count () ) ) ;
            return true ;
        }
        else
            return false ;

    }
    else if ( event->type () == QEvent::DragEnter )
    {
        QDragEnterEvent * ke = static_cast< QDragEnterEvent * > ( event ) ;
        if ( watched->property( SEL_SQUARES_MARK ).toBool () == true )
            return true ;
        QWidget * w = qobject_cast< QWidget * > ( watched ) ;
        if ( w )
        {
            dragEnterEvent ( new QDragEnterEvent ( ke->pos () + w->pos () ,
                                                 ke->dropAction () ,
                                                 ke->mimeData () ,
                                                 ke->mouseButtons () ,
                                                 ke->keyboardModifiers () ) ) ;
        }
        return true ;

    }
    else if ( event->type () == QEvent::MouseButtonDblClick )
    {
        QWidget * w = qobject_cast< QWidget * >  ( watched ) ;
        if ( w )
        {
            QString signalSignature = "default here";

#ifndef Q7BASIC // activate for testing purpose only
            QMessageBox msgBox ;
            msgBox.setText (QString( "on_%1_%2" ).arg( w->objectName() ).arg( signalSignature ));
            msgBox.exec () ;
#endif
            GLOBAL_Q7BFormDesigner->emitSignalToIDE_AddSignal(QString( "on_%1_%2" ).arg( w->objectName() ).arg( signalSignature ));

            emit widgetsSelected ( QList< QPointer < QWidget > > () << w ,
                                  false ) ;
        }
        return true ;
    }
    else if ( event->type () == QEvent::ToolTip )
    {
        QWidget * w = qobject_cast< QWidget * >  ( watched ) ;
        if ( w->property( SEL_SQUARES_MARK ).toBool () == true )
            return true ;
        if ( w )
            emit widgetTooltip( w ) ;
        return true ;
    }
    return false ;
}

void DesigningArea::keyPressEvent  ( QKeyEvent * event )
{
    if ( mode )
        return ;
    if ( event->modifiers ().testFlag ( Qt::ShiftModifier ) )
    {
        QPoint p ;
        if ( event->key () == Qt::Key_Right )
            p = QPoint ( arrowResizingDelta , 0 ) ;
        else if ( event->key () == Qt::Key_Left )
            p = QPoint ( - arrowResizingDelta , 0 ) ;
        else if ( event->key () == Qt::Key_Up )
            p = QPoint ( 0 , - arrowResizingDelta ) ;
        else if ( event->key () == Qt::Key_Down )
            p = QPoint ( 0 , arrowResizingDelta ) ;
        if ( p !=  QPoint () )
            emit resizeWidgets ( pointToGrid ( p ) , 4 ) ;
    }
    else
    {
        int step = gridDelta ;
        if ( event->modifiers ().testFlag ( Qt::ControlModifier ) )
            step = arrowMovingDelta ;
        QPoint p ;
        if ( event->key () == Qt::Key_Right )
            p = QPoint ( step , 0 ) ;
        else if ( event->key () == Qt::Key_Left )
            p = QPoint ( - step , 0 ) ;
        else if ( event->key () == Qt::Key_Up )
            p = QPoint ( 0 , - step ) ;
        else if ( event->key () == Qt::Key_Down )
            p = QPoint ( 0 , step ) ;

        if ( p !=  QPoint () )
            emit movingWidgets ( pointToGrid ( p ) ) ;
    }
}

void DesigningArea::keyReleaseEvent ( QKeyEvent * event )
{
    if ( mode )
        return ;
    if ( ! event->isAutoRepeat ()
            && event->modifiers ().testFlag ( Qt::ControlModifier )
            && ( event->key () == Qt::Key_Down
            || event->key () == Qt::Key_Right
            || event->key () == Qt::Key_Up
            || event->key () == Qt::Key_Left ) )
        emit endMoving () ;
    else if ( ! event->isAutoRepeat ()
             && event->modifiers ().testFlag ( Qt::ShiftModifier )
             && ( event->key () == Qt::Key_Down
             || event->key () == Qt::Key_Right
             || event->key () == Qt::Key_Up
             || event->key () == Qt::Key_Left ) )
        emit endResizing () ;
}

void DesigningArea::mouseDoubleClickEvent ( QMouseEvent * event )
{
    QFrame::mouseDoubleClickEvent ( event ) ;

}

QRect DesigningArea::rectToGrid ( QRect r )
{
    QPoint p = pointToGrid ( r.topLeft () ) ;
    return QRect ( p.x() , p.y() ,
                   r.width() - r.width() % gridDelta ,
                   r.height() - r.height() % gridDelta ) ;
}

void DesigningArea::emitWidthSmall ( int d )
{
    emit widthSmall ( d ) ;
}

void DesigningArea::emitHeightSmall ( int d )
{
    emit heightSmall ( d ) ;
}

void DesigningArea::emitGeomSmall ( int h , int w )
{
    emit geomSmall ( h , w ) ;
}

void DesigningArea::drawFocus ( bool b )
{
    Q_UNUSED ( b )
    /*if ( b )
        setLineWidth ( 3 ) ;
    else
        setLineWidth ( 1 ) ;*/
}

void DesigningArea::setShowCursorPos ( bool b )
{
    showCursorPos = b ;
}

bool DesigningArea::getShowCursorPos (  )
{
    return showCursorPos ;
}

void DesigningArea::setMode ( bool b )
{
    mode = b ;
}

QWidget * DesigningArea::currentWidget ()
{
    return movingWidget ;
}


