/*
    This file containes declaration of classes, which provides designing forms.
    Form is a widget, which containes one or more DesigningArea widgets.
*/

#ifndef FORMMAKET_H
#define FORMMAKET_H

#include <QWidget>
#include <QMdiSubWindow>
#include <QPointer>
#include <QPropertyAnimation>
#include <QTabWidget>
#include <QParallelAnimationGroup>
#include <QScrollArea>

//
/*
    Abstract class for all designing forms.
    Provides form resizing, initialization of DesigningArea objects,
    and multiplexing signals of DesigningArea objects.
*/
//class FormMaket : public QWidget
class DesigningArea ;
//class FormMaket : public QMdiSubWindow
class FormMaket : public QWidget
{
    Q_OBJECT
    // List of all contained DesigningArea objects.
    QList < DesigningArea * > areas ;
    // Creating connections of areas signals
    void connectAreas () ;
    bool isInit ;
    // Animations are used for smooth changing of sections size
    QParallelAnimationGroup * animationGroup ;

    bool gridEnabled ;

public :
    enum SectionsSizePolicy
    {
        FIXED,
        AUTO
    } ;

    explicit FormMaket  ( QWidget *parent ) ;
    // Controls grids of all DesigningArea objects.
    void setGrid ( bool b ) ;
    // Controls cursor pos tooltip showing of all DesigningArea objects.
    void setShowCursorPos ( bool b ) ;
    // returns cursor showing mode
    bool getShowCursorPos ( ) ;
    // Initializes
    void init () ;
    // Returns all sections of this maket
    QList < DesigningArea * > getAreas () ;
    // Returns section by its name
    DesigningArea * sectionByName ( QString n ) ;
    // Returns name of section
    QString nameOfSection ( DesigningArea * n ) ;
    // Returns grid state of sections
    bool gridState (  ) ;
    // Changes visibility of section
    virtual void setAreaVisible ( QString name , bool b ) ;
    // Returns visibility state of sections
    bool isAreaVisible ( QString name ) ;
    // Sets size changing policy of sections
    void setSectionsSizePolicy ( SectionsSizePolicy p ) ;
    // Returns true if sections are autoresizable
    bool isSectionsAutoResizeble () ;

    // returns grid step
    int getGridStep () ;

    virtual QVariantMap splitterPositions ( ) ;
    virtual void setSplitterPositions ( QVariantMap ) ;
protected :
    virtual void createAreas () = 0 ;
    // Adds area to area list
    void addArea ( DesigningArea * ) ;
    // Stores current size policy of sections
    SectionsSizePolicy policy ;
    void closeEvent ( QCloseEvent *) ;
signals :
    // Emitted when widgets on one of sections are moved
    void movingWidgets ( QPoint ) ;
    // Emitted when widgets on one of sections are resized
    void resizeWidgets ( QPoint , int ) ;
    // Emitted when moving series has ended
    void endMoving () ;
    // Emitted when resizing series has ended
    void endResizing () ;
    // Emitted when section gets a focus
    void inFocus ( DesigningArea * ) ;

    //void dragWidget () ;
    // Emitted when user drops widgets on one of sections.
    void moveWidgetByPaste ( QWidget * parent , QPoint hotspot) ;
    // Emitted when user executes context menu (makes right-click) for one of sections.
    void contextMenuSignal ( QWidget * ) ;
    // Emitted when user drags widgets on one of sections.
    void widgetsDragged ( QWidget * ) ;
    // Emitted when user selects widgets on one of sections.
    void widgetsSelected ( QList < QPointer < QWidget > > list , bool withControl) ;
    // Emitted when user clicks on free space on one of sections
    void clearSelection () ;
    // Emitted when it is time to show tooltip for widget
    void widgetTooltip( QWidget * w ) ;

    void closing () ;

    void leftClicked () ;

public slots :
    // Called when Maket should be resized
    virtual void enlargeGeom ( int w , int h ) ;
    // Called when sections sizes must be recalculated
    virtual void updateSectionSize (DesigningArea*) ;
    void setMode ( bool b ) ;
private slots :
    // Called when a section gets the focus
    void focusChanged ( DesigningArea * ) ;
} ;

/*
    Provides form with three sections  (header, body and footer).
*/
class QResizebleSplitter ;
class ThreeSectionsMaket : public FormMaket
{
    Q_OBJECT
    void createAreas () ;
    QResizebleSplitter * spl ;
    DesigningArea * daBody ;
    DesigningArea * daHeader ;
    DesigningArea * daFooter ;

    QScrollArea * headerScrollArea ;
    QScrollArea * footerScrollArea ;
    QScrollArea * bodyScrollArea  ;
    void resizeEvent (QResizeEvent *) ;
public:
    explicit ThreeSectionsMaket ( QWidget * parent = 0 ) ;
    void setAreaVisible ( QString name , bool b ) ;

    QVariantMap splitterPositions ( ) ;
    void setSplitterPositions ( QVariantMap ) ;
private slots :
    void splitterMoved ( int , int ) ;
public slots :
    void updateSectionSize (DesigningArea*) ;
} ;

#endif // FORMMAKET_H
