/* This file containes declarations of helper functions for working with
  QObject-inherited object properties
*/
#ifndef PROPS_FUNCTIONS_H
#define PROPS_FUNCTIONS_H
#include <QObject>
#include <QVariant>
namespace PF
{

    // Adds special info in dedicated dynamic property, which marks property "name" as read-only
    void setPropReadOnly ( QObject * ob , QString name ) ;
    bool checkPropIsInternal ( QString name ) ;
    // Checks special info in dedicated dynamic property, if it marks property "name" as read-only
    bool checkPropIsReadOnly ( QObject * ob , QString name ) ;
    bool checkPropIsReadOnly ( QObject * ob , int index ) ;
    // Returns visible in PropertyEditor dynamic props count
    int visibleDynPropsCount ( QObject * ob ) ;
    // Returns dynamic prop name
    QString getVisibleDynPropName ( int index , QObject * ob ) ;

    QVariant readMetaProperty ( const QObject *w , QString name ) ;
    QVariant readMetaProperty ( QObject * w , int index ) ;
    bool writeMetaProperty( QObject * w , QString name , QVariant value ) ;
    bool writeMetaProperty( QObject * w , int index , QVariant value ) ;
    int metaPropertyCount( QObject * w ) ;
    int metaPropertyIndex( QObject * w , QString name ) ;
    QString metaPropertyName( QObject * w , int index ) ;
    QVariant::Type metaPropertyType ( QObject * w , QString name ) ;
    QVariant::Type metaPropertyType ( QObject * w , int index ) ;
    bool writeProperty ( QObject * w , QString name , QVariant value ) ;
    QVariant readProperty ( const QObject *w , QString name ) ;
    bool checkMetaPropertyWritable ( QObject * w , QString name ) ;
    bool checkMetaPropertyWritable ( QObject * w , int index ) ;
    bool checkPropertyWritable ( QObject * w , QString name ) ;
    bool checkPropertyWritable ( QObject * w , int index ) ;
    QList<QPair<QString, QStringList> > groupedProperties( const QObject *w  ) ;
    bool hasReplacement ( QObject *w, QString name ) ;
    bool writeMetaPropertyD( QObject * w , QString name , QVariant value ) ;
}



#endif // PROPS_FUNCTIONS_H
